#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <Xmt/Xmt.h>
#include <Xmt/Dialog.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Procedures.h>
#include <Xmt/Layout.h>
#include <Xm/List.h>

#include <linux/isdn.h>

#define CHANL	"*channel%d"
#define LSTAT	"*line_stat%d"
#define CNUM	"*call_number%d"
#define CTYPE	"*call_type%d"

#define ISDNINFO	"/dev/isdninfo"

static char *fallb_rsrcs[]= {
#include "misdnstat.ad.h"
	NULL
};

static Widget toplevel;
static XtInputId	xinid;
static FILE	*info_fp;
char *idmap_line;
char *chmap_line;
char *drmap_line;
char *usage_line;
char *flags_line;
char *phone_line;

void
set_bg_color(char *w_name, char *color)
{
	Widget w;

	if (w=XmtNameToWidget(toplevel,w_name))
		XmtSetValue(w, "background", color);
}

void
set_w_text(char *w_name, char *s)
{
	Widget w;

	if (w=XmtNameToWidget(toplevel,w_name))
		XmtSetValue(w, "labelString", s);
}

void
set_fields(int line,char *type,char *phone,char *status,char *usage,char *channel)
{
	char dummy[40];

	sprintf(dummy,LSTAT,line);
	set_w_text(dummy,status);
	set_bg_color(dummy,type);

	sprintf(dummy,CHANL,line);
	set_w_text(dummy,channel);

	sprintf(dummy,CNUM,line);
	set_w_text(dummy,phone);

	sprintf(dummy,CTYPE,line);
	set_w_text(dummy,usage);
}

/* Shamelessly stolen (and fixed) from imon */
static void
check_info()
{

	char *ptr_idmap;
	char *ptr_chmap;
	char *ptr_drmap;
	char *ptr_usage;
	char *ptr_flags;
	char *ptr_phone;

	char channel[30];
	char status[10];  
	char phone[30];
	char usage[10];
	char type[10];  

	char temp[20];   

	int  local=0;
	int  chanum;
	int  line;

          
	fgets(idmap_line, 4095, info_fp);
	fgets(chmap_line, 4095, info_fp);
	fgets(drmap_line, 4095, info_fp);
	fgets(usage_line, 4095, info_fp);
	fgets(flags_line, 4095, info_fp);
	fgets(phone_line, 4095, info_fp);

	ptr_idmap = idmap_line + 7;
	ptr_chmap = chmap_line + 7;
	ptr_drmap = drmap_line + 7;
	ptr_usage = usage_line + 7;
	ptr_flags = flags_line + 7;
	ptr_phone = phone_line + 7;

	for(line=1; line<3; line++) {

		sscanf(ptr_idmap, "%s", channel);
		ptr_idmap = ptr_idmap + strlen(channel) + 1;
		sscanf(ptr_usage, "%d", &local);   
		sscanf(ptr_usage, "%s", temp);
		ptr_usage = ptr_usage + strlen(temp) + 1;
		sscanf(ptr_phone, "%s", phone);
		ptr_phone = ptr_phone + strlen(phone) + 1;
		sscanf(ptr_chmap, "%d", &chanum);
		sscanf(ptr_chmap, "%s", temp);
		ptr_chmap = ptr_chmap + strlen(temp) + 1;


		if (chanum==-1)
			continue;    

		if ((local&7) == 0) {
			/* ptr_phone = ptr_phone + 4; */
			sprintf(status, "%s",(local&64)?"Excl.":"Offline");
			sprintf(phone, "-------------");
			sprintf(usage, "-----");
			sprintf(type, "green");
		} else {
			sprintf(status, "Online");
			sprintf(type, (local&ISDN_USAGE_OUTGOING)?"red":"yellow");
			switch (local&7) {
				case ISDN_USAGE_RAW:
					sprintf(usage, "Raw");
					break;
				case ISDN_USAGE_MODEM:
					sprintf(usage, "Modem");
					break;
				case ISDN_USAGE_NET:
					sprintf(usage, "Net");
					break;
				case ISDN_USAGE_VOICE:
					sprintf(usage, "Voice");
					break;
				case ISDN_USAGE_FAX:
					sprintf(usage, "Fax");
					break;
				default:
					sprintf(usage, "");
					break;
			}
		}
		set_fields(line-1,type,phone,status,usage,channel);
	}
}

void
open_iinfo(XtAppContext app)
{
	if ((info_fp=fopen(ISDNINFO,"r")) == NULL) {
		perror(ISDNINFO);
		exit(1);
	}
	xinid=XtAppAddInput(app, fileno(info_fp),
		 (XtPointer) XtInputReadMask,
         (XtInputCallbackProc) check_info, (XtPointer) NULL);
}

void
quitting()
{
	XtRemoveInput(xinid);
	fclose(info_fp);
	free(idmap_line);
	free(chmap_line);
	free(drmap_line);
	free(usage_line);
	free(flags_line);
	free(phone_line);
	exit(0);
}

void
set_all_up(XtAppContext app)
{
	if ((idmap_line=(char *)malloc(4096))==NULL) exit(1);
	if ((chmap_line=(char *)malloc(4096))==NULL) exit(1);
	if ((drmap_line=(char *)malloc(4096))==NULL) exit(1);
	if ((usage_line=(char *)malloc(4096))==NULL) exit(1);
	if ((flags_line=(char *)malloc(4096))==NULL) exit(1);
	if ((phone_line=(char *)malloc(4096))==NULL) exit(1);
	open_iinfo(app);
}

main(int argc, char **argv)
{
    XtAppContext app;
    
    toplevel = XtAppInitialize (&app, "misdnstat", NULL, 0,
                    &argc, argv, fallb_rsrcs, NULL, 0);

	XmtRegisterLayoutParser();

	XmtRegisterWidgetConverter();
	XmtRegisterCallbackConverter();
	XmtRegisterXmStringConverter();
	XmtRegisterPixmapConverter();
	XmtRegisterBitmapConverter();
	XmtRegisterMotifWidgets();
	XmtRegisterXmtWidgets();
	XmtRegisterXtProcedures();
	XmtRegisterXmtProcedures();

	XmtCreateChildren(toplevel);
	XtRealizeWidget (toplevel);
    XmtAddDeleteCallback(toplevel,XmDO_NOTHING,quitting,NULL);
    XmtVaRegisterCallbackProcedures(
	      "quitting", quitting, XmtRCallbackUnused,
	      NULL );

    set_all_up(app);

    XtAppMainLoop(app);
}
